package ucot;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

import ucot.exceptions.BadPropertyValueException;
import ucot.utils.PropertiesTools;

/**
 * <pre>public abstract class <b>ModuleProperties</b><br />implements ModulePropertyInterface</pre>
 * 
 * <p>This class implements abstracts methods for handling,
 * saving and loading of module's properties. It is recommended
 * that all UCOT modules extend from this class because all the
 * interfaces require these methods to be implemented anyway and
 * these methods implemented here are sufficient for practically
 * all other modules besides the core.</p>
 * 
 * @see #ucot.ModulePropertyInterface
 * @author tujupien
 */
public abstract class ModuleProperties implements ModulePropertyInterface {
	
	protected URL propertiesURL = getPropertiesURL();
	protected Properties properties = loadDefaultProperties();
	
	/**
	 * <p>Method for creating the URL from the properties file, which
	 * is the same as the class name with an .xml extension.</p>
	 * 
	 * @return URL to the properties file.
	 */
	private URL getPropertiesURL() {
		String filename = this.getClass().toString();
		filename = filename.substring(filename.lastIndexOf('.') + 1) + ".xml";
		File propertiesFile = new File(filename);
		try {
			propertiesFile.createNewFile();
		} catch (IOException e) {}
		
		try {
			return propertiesFile.toURL();
		} catch (MalformedURLException e) {
			return null;
		}
	}

	/**
	 * @see ucot.ModulePropertyInterface#getProperties()
	 */
	public synchronized Properties getProperties() {
		return properties;
	}

	/**
	 * @see ucot.ModulePropertyInterface#setProperties(java.util.Properties)
	 */
	public synchronized void setProperties(Properties properties) {
		this.properties = PropertiesTools.merge(properties, this.properties);
	}

	/**
	 * @see ucot.ModulePropertyInterface#applyProperties()
	 */
	public synchronized void applyProperties() throws BadPropertyValueException {}

	/**
	 * @see ucot.ModulePropertyInterface#saveProperties()
	 */
	public synchronized void saveProperties() throws IOException {
		PropertiesTools.saveProperties(properties, this.propertiesURL);
	}

	/**
	 * @see ucot.ModulePropertyInterface#loadProperties()
	 */
	public synchronized void loadProperties() throws IOException {
		properties = PropertiesTools.merge(PropertiesTools.loadProperties(
				loadDefaultProperties().propertyNames(),
				this.propertiesURL), properties);
	}

	/**
	 * @see ucot.ModulePropertyInterface#loadDefaultProperties()
	 */
	public Properties loadDefaultProperties() {
		return new Properties();
	}

}
