package ucot.core;

import java.io.IOException;
import java.net.URL;
import java.util.Vector;

import ucot.ModulePropertyInterface;
import ucot.heuristic.HeuristicInterface;
import ucot.input.InputCollection;
import ucot.input.UseCase;
import ucot.input.UseCaseCollection;
import ucot.model.AnalyzeModel;
import ucot.output.OutputInterface;
import ucot.parser.ParserInterface;

/**
 * <pre>public interface <b>ControlInterface</b><br />extends ModulePropertyInterface</pre>
 * 
 * <p>This interface controls the basics of the UCOT-program
 * (starting, parsing, shutting down etc). UCOT core implements
 * this interface and it is designed in a way that the user
 * interface using the core can control the execution of the
 * program effectively.</p>
 * 
 * @see #ucot.ModulePropertyInterface
 * @author UCOT
 */
public interface ControlInterface extends ModulePropertyInterface {
	public static final String PROPERTY_CURRENT_PARSER = "CURRENT_PARSER";
	public static final String PROPERTY_CURRENT_HEURISTIC = "CURRENT_HEURISTIC";
	
	/**
	 * <p>Shuts down the program. The shutdown routine triggers
	 * all possible autosave actions and after that the core
	 * gets rid of all its modules.</p>
	 */
	public void shutdown();	
	
	/**
	 * <p>Clears the whole current analyze model. Basically this
	 * is similiar to creating a whole new empty analyze model.</p>
	 */
	public void clearAnalyzeModel();
	
	/**
	 * <p>Loads use cases from file.</p>
	 * 
	 * @param url URL of the file.
	 */
	public void loadUseCases(URL url);
	
	/**
	 * <p>Reloads use cases from file. First core
	 * should remove all use cases that are loaded
	 * from given url. Then it should read use cases
	 * from the file.</p>
	 * 
	 * @param url URL of the file to reload.
	 */
	public void reloadUseCases(URL url);
	
	
	/**
	 * <p>Loads serialized analyze model from a file.</p>
	 * 
	 * @param url URL of the file containing analyze model.
	 * @throws IOException If something goes wrong with loading
	 *   use the analyze model from given URL.
	 */
	public void loadAnalyzeModel(URL url) throws IOException;
	
	/**
	 * <p>Saves analyze model to a java serialization file.</p>
	 * 
	 * @param url Target file.
	 */
	public void saveAnalyzeModel(URL url);
	
	/**
	 * <p>Returns all loaded use cases.</p>
	 * 
	 * @return Use case colletion.
	 */
	public UseCaseCollection getUseCaseCollection();
	
	/**
	 * <p>Requests core to parse use cases, perform
	 * heuristic on them and add them to current analyze model.</p>
	 * 
	 * <p>Default parser and heuristic are used for this operation.</p>
	 * 
	 * @param useCases Use cases to work magic on.
	 */
	public void addToAnalyzeModel(Vector<UseCase> useCases);
		
	/**
	 * <p>Requests core to parse use cases, perform heuristic
	 * on them and add them to current analyze model.</p>
	 * 
	 * <p>If either the given parser or heuristic is unknown
	 * to the core, then nothing is done.</p>
	 * 
	 * @param useCases Use cases to work magic on.
	 * @param parser Parser to use.
	 * @param heuristic Heuristic to use.
	 */
	public void addToAnalyzeModel(Vector<UseCase> useCases, ParserInterface parser, HeuristicInterface heuristic);
	
	
	/**
	 * <p>Returns analyze model from core.</p>
	 * 
	 * @return Current analyze model.
	 */
	public AnalyzeModel getAnalyzeModel();	
	
	/**
	 * <p>Returns a vector of parsers that are available.</p>
	 * 
	 * @return Vector containing all known parser adapters.
	 */
	public Vector<ParserInterface> getParsers();
	
	/**
	 * <p>Returns a list of heuristics that are available.</p>
	 * 
	 * @return Vector containing all known heuristic adapters.
	 */
	public Vector<HeuristicInterface> getHeuristics();
	
	/**
	 * <p>Returns a list of output adapters available.</p>
	 * 
	 * @return Vector containing all known output adapters.
	 */
	public Vector<OutputInterface> getOutputs();
	
	/**
	 * <p>Returns the InputCollection.</p>
	 * 
	 * @return InputCollection which includes all known input adapters.
	 */
	public InputCollection getInputs();
	
	/**
	 * <p>Sets the default parser to use.</p>
	 * 
	 * @param parser Parser to be used by default.
	 */
	public void setCurrentParser(ParserInterface parser);
	
	/**
	 * <p>Sets the default heuristic to use.</p>
	 * 
	 * @param heuristic Default heuristic to be used.
	 */
	public void setCurrentHeuristic(HeuristicInterface heuristic);
	
	/**
	 * <p>Returns the current default parser.</p>
	 * 
	 * @return Default parser.
	 */
	public ParserInterface getCurrentParser();
	
	/**
	 * <p>Returns the current heuristic.</p>
	 * 
	 * @return Current heuristic.
	 */
	public HeuristicInterface getCurrentHeuristic();
	
	/**
	 * <p>Exports given analyze model to given url using given output adapter.</p>
	 * 
	 * @param url Destination URL.
	 * @param output Output adapter to use.
	 * @param analyzeModel Analyze model to export.
	 * @throws If something goes wrong with the output, then an
	 *   exception is thrown.
	 */
	public void output(URL url, OutputInterface output, AnalyzeModel analyzeModel) throws Exception;
	
}
