package ucot.input;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;
import java.util.logging.Logger;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import ucot.ModuleProperties;
import ucot.exceptions.BadPropertyValueException;
import ucot.exceptions.FileFormatNotSupportedException;
import ucot.utils.StringTools;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;


/**
 * This InputInterface reads ProcessMl-files and parses usecases from them.
 * References to sub-usecase is stored to usecase's steps. If sub-usecase has no
 * references to it, it is currently discarded. 
 * @author vevijopi
 *
 */
public class ProcessMlInputAdapter extends ModuleProperties implements InputInterface {
	private Logger logger = Logger.getLogger("ucot.parser");
	public UseCaseCollection read(URL url) throws IOException  {
		
		if (url == null) throw new IOException();
		
		UseCaseCollection collection = new UseCaseCollection();
		
		UseCase usecase = null;		
		File xmlFile = new File(url.getFile());
		
		DOMParser parser = new DOMParser();
		InputSource input = new InputSource(new FileInputStream(xmlFile));
		
		//Try parsing the xml file 
		try {
		parser.parse(input);
		}catch(SAXException e) {						
			//TODO localize me
			throw new FileFormatNotSupportedException("Fileformat not supported");
		}
		// Reading parsed data.
		Document doc = parser.getDocument();
		Element root = doc.getDocumentElement();
		
		//processInstance elements are the ones that store usecase data in processml
		NodeList list = root.getElementsByTagName("processInstance");
		
		Element processInstance = null;
		Element abstractionElement = null;
		NodeList abstractionlist = null; 
		NodeList stepList = null;
		
		//If the xml file doesn't contain any processInstances, it contains no usecases
		if (list.getLength() < 1) 
			throw new FileFormatNotSupportedException();
		
		//Now go through all the processInstances (read: Usecases)
		for(int i=0;i<list.getLength();i++) {
			//Create a new usecase
			usecase = new UseCase();
			usecase.setUrl(url);
			
			//Get the next processInstance element from the xml
			processInstance = (Element)list.item(i);			
		
			//Get the value of the processInstance, which is the name of usecase
			usecase.setName(processInstance.getFirstChild().getNodeValue().trim());			
			
			//Get all abstractions from current processinstance
			abstractionlist = processInstance.getElementsByTagName("abstraction");			
			
			//Find the one with level=0, we don't have to implement other abstraction levels
			abstractionElement = getCorrectAbstraction(abstractionlist);
			
			//If no correct element found, then ignore this processInstance
			if (abstractionElement == null) {
				//TODO localize me
				logger.fine("No abstract-element found, skipping this processInstance!");				
				continue;
			}
			
			//Get all steps within this abstraction
			stepList = abstractionElement.getElementsByTagName("step");
			
			
			//If something went wrong when parsing the steps, ignore this usecase
			if (!ParseSteps(usecase,stepList)) {
				//TODO localize me
				logger.fine("Failed to parse steps!");
				continue;
			}
			
			//And finally if everything went ok, add the usecase to our collection
			collection.addUseCase(usecase);
			
			//Parse usecases id and store it for later sub-usecase sorting purposes
			ParseInstanceDetails(processInstance,usecase);			
		}
		
		//Finally when all usecases are parsed we make appropriate sub-usecase references
		//to (main) usecase steps and then remove the usecases from the main usecasecollection
		//so they wont be confused with main usecases
		collection.resolveSubUseCases(true);		
		return collection;
	}
	
	/**
	 * Goes throguh a nodelist containing "abstraction"-named elements and returns the one
	 * which has level 0 
	 * @param abstractions list of elements with the name abstraction (from processml) 
	 * @return abstraction-element with level 0, or null if none found
	 */
	private Element getCorrectAbstraction(NodeList abstractions) {		
		Node level = null;
		
		for(int i=0;i<abstractions.getLength();i++) {
			//Find element called level			
			level = abstractions.item(i).getAttributes().getNamedItem("level");
			
			//check that it really was found
			if (level == null) {
				//TODO localize me
				logger.fine("No level attribute found from abstract-element");
				continue;				
			}
			
			//If it has value 0, then its the one we're looking for
			if (level.getNodeValue().equals("0")) 
				return (Element)abstractions.item(i);			
		}
		
		//if none with level=0 were found, return nothing. This could be changed to abstractions.item(0)
		//if its a common convention to not set level attribute if there is only 1 abstraction element
		return null;
	}
	
	/**
	 * Parses steps from given nodelist, stores them to given usecase
	 * @param usecase where steps are stored
	 * @param steps nodelist containing the "step"-elements
	 * @return true if everything went ok
	 */
	private boolean ParseSteps(UseCase usecase, NodeList steps) {
		//Sanity checks
		if (usecase == null || steps == null) 
			return false;
		if (steps.getLength() < 1) 
			return false;
		
		UseCaseStep step = null;
		
		//Go through all items
		for(int i=0;i<steps.getLength();i++) {
			//Parse individual steps
			step = ParseStep(steps.item(i));
			
			//if we've got a step, add it to our usecae			 
			usecase.addStep(step);
		}
		
		//Everything went ok, return true
		return true;
	}	
	
	/**
	 * Parses usecase step from given node and returns it, or null if acceptable one wasn't found
	 * @param node xml-element that contains a step
	 * @return parsed usecasestep or null
	 */
	private UseCaseStep ParseStep(Node node) {
		if (node == null)
			return null;
		
		//If its a sub-step, we don't want it
		if (node.getParentNode().getNodeName().equals("step")) 
			return null;
		
		//remove all line breaks,tabs and whitespaces
		String tmp = StringTools.removeWhitespacesTabsLineBreaks(node.getTextContent().trim());
		
		//Create a new usecase step, no relation to another subusecase
		//if there will be a relation, it will be added to this later
		UseCaseStep step = new UseCaseStep(tmp,null);
		
		//Check if it has a target, a sub-usecase
		Node target = node.getAttributes().getNamedItem("target");
		
		//if there was no target, we can return the step already
		if (target == null)
			return step;		
		
		//Otherwise we store the target's id to the step for later usage
		step.setSubUseCaseId(target.getNodeValue());
		
		return step;
	}
	
	
	
	/**
	 * Parses processInstance's id and adds it to usecase, also checks if this usecase is a subusecase
	 * @param processInstance xml-element to parse the details from
	 * @param usecase usecase that was created from processInstance attribute
	 */
	private void ParseInstanceDetails(Element processInstance, UseCase usecase) {		
		usecase.setId(processInstance.getAttribute("id"));
		
		/* Miika told that this isn't a reliable attribute
		if (processInstance.getAttribute("type").equalsIgnoreCase("sub"))
			usecase.setAsSubUseCase(true);
		*/
	}

	/**
	 * Tests if this adapter can read the file. For now, only test is that the file ends with ".xml"
	 * If we're adding more xml based inputs, we could verify that the file matches processml.dtd.
	 * I didn't implement this, because it could slow down loading (other types of) files a bit.
	 * 
	 * @param url url of the file to test
	 * @return true if this adapter can read the file
	 */
	public boolean canRead(URL url) {
		if (url.getFile().toLowerCase().endsWith(".xml")) 
			return true;
		return false;
	}
	
	/**
	 * Prints a nodelist, for testing purposes nly..
	 * @param a nodelist to print
	 */
	public void printNodeList(NodeList a) {
		if (a == null) 
			return;
		for(int i=0;i<a.getLength();i++) {
			System.out.println(a.item(i).getNodeName());
			
		}
	}

	/**
	 * Test program used while programming this class
	 * @param args
	 */
	public static void main(String args[]) {
		ProcessMlInputAdapter test = new ProcessMlInputAdapter();
		URL url = null;
		UseCaseCollection collection = null;
		
		try {
		url = new URL("file:///C:/MyTemp/vevijopi/UCOT/h_lausuntopyynto_laitokset.xml");
		} catch(MalformedURLException e) {
			System.out.println("Ei toimi");
		}
		try {
			collection = test.read(url);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		System.out.println(collection);
	}
    
    /**
     * returns inputs name
     */
    public String toString() {
        return "ProcessMLInputAdapter";
    }

	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#applyProperties()
	 */
	public void applyProperties() throws BadPropertyValueException {
		// TODO Auto-generated method stub
		
	}
	
	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#loadDefaultProperties()
	 */
	public Properties loadDefaultProperties() {
		Properties result = super.loadDefaultProperties();
		// TODO Set default properties here, for example:
		// result.setProperty("SETTING_1", "Woot");
		// result.setProperty("SETTING_2", "WTF?");
		return result;
	}

}
