package ucot.input;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Properties;

import ucot.ModuleProperties;
import ucot.exceptions.BadPropertyValueException;


/**
 * Reads usecases that are stored in this format:
 * [name]
 * Name of the usecase is here
 * [id]
 * Id of the usecase (must be unique within file, it is discarded after loading) 
 * [steps]
 * Steps separated with linebreak. Step can have a sub-usecase, it is marked with (usecaseid) after the steps description and "."-character.
 * [end]
 * Use case ends with [end] tag, another usecase can begin with [name] element now
 * 
 * @author pajumasu & vevijopi
 *
 */
public class SimpleInputAdapter extends ModuleProperties implements InputInterface {
	
	public static final String name = "Simple input adapter";
	
	public String getName() {
		return name;
	}
	
	public UseCaseCollection read(URL url) throws IOException {
		UseCaseCollection collection = new UseCaseCollection();
		BufferedReader reader = null;
		try {
							
			reader = new BufferedReader(new InputStreamReader(url.openStream()));
			readUseCases(reader, collection, url);
			collection.resolveSubUseCases(true);
		} finally {
			reader.close();
		}
		return collection;
	}

	/**
	 * Reads usecases from given reader, stores them to given collection and sets their url to given one.
	 * @param reader reader for the inputstream
	 * @param collection where all found usecases are stored
	 * @param url url for the usecase
	 * @throws IOException 
	 */
	public void readUseCases(BufferedReader reader, UseCaseCollection collection, URL url) throws IOException {		
		UseCase usecase = null;
		
		while(reader.ready()) {
			//Read a usecase
			usecase = readUseCase(reader,url);
			//then add it
			collection.addUseCase(usecase);						
		}
	}
	
	/**
	 * Reads a single usecase from given reader, sets it's url to given on
	 * @param reader reader to read usecases with
	 * @param url url for the usecases
	 * @return read new usecase or null if error encountered
	 * @throws IOException
	 */
	public UseCase readUseCase(BufferedReader reader, URL url) throws IOException {
		UseCase usecase = new UseCase();
		boolean inside = false; // Are we inside usecase.
		boolean insideSteps = false; 
		boolean nextIsName = false;		
		boolean nextIsId = false;
		
		
		while(reader.ready()){
			//Mark the position for "undo"
			reader.mark(500);
			
			//Read a line
			String line = reader.readLine().trim();			
			
			//If next should be the usecase name
			if (nextIsName && line.length() > 0){
				
				//Name can't have brackets
				if (line.startsWith("[")) {
					//reset reader to the mark before reading this line
					//if in case the current usecase was improperly formed and
					//a new one begins here
					reader.reset();
					return null;					
				}
				else {
					usecase.setName(line);		
					nextIsName = false;					
				}
				
				continue;
			}
			
			//If next should be id
			if (nextIsId && line.length() > 0){
				
				//But the line starts with [ then the usecase isn't properly formed
				if (!line.startsWith("[")) {				
					usecase.setId(line);
				} else {
//					reset reader to the mark before reading this line
					//if in case the current usecase was improperly formed and
					//a new one begins here
					reader.reset();
					
					//And we return null
					return null;
				}
				
				nextIsId = false;
				continue;
			}
			
			
			//or hasn't the usecase even started yet
			if (!inside){
				//If there is [name], usecase's name should be on the next line
				nextIsName = line.trim().equals("[name]");
				
				//if [name] is found, then the usecase has started
				if (nextIsName) inside = true;
			} else {
				//if [steps] found, then the steps are found next line
				if (line.equals("[id]") && !insideSteps) {
					nextIsId = true;
				} else 
				if (line.equals("[steps]")){
					insideSteps = true; 
					//If [end] is found
				} else if (line.equals("[end]")){
					//Then append the usecase to our collection
					usecase.setUrl(url);
					
					//Usecase should have steps, or its not really a usecase
					return usecase;
				} else if (insideSteps){
					//If we are inside the usecase, name has been found but no [end]
					//then its a step for the usecase
					
					//But first check that it hasn't got braces
					if (line.startsWith("[")) {
						//reset reader to the mark before reading this line
						//if in case the current usecase was improperly formed and
						//a new one begins here
						reader.reset();
						
						return null;
					}
					
					usecase.addStep(parseStep(line));
				}
			}
		}
		return usecase;		
		
	}
	
	
	/**
	 * Parses usecase step from given line. Also stores relation id
	 * @param line
	 * @return parsed usecasestep
	 */
	private UseCaseStep parseStep(String line) {
		if (line == null)
			return null;
		
		UseCaseStep step = new UseCaseStep(line, null);
		
		//Usecase step should end with "." and possible id should come after it
		int dot = line.lastIndexOf(".");		
        //find () within the line
		int end = line.lastIndexOf(")");
		int begin = line.lastIndexOf("(") + 1;		
		
		
		
		//Sanity check for the () characters and check that if they even exist there
		//and that the () isn't before ".", because then it could just be part of the usecase
		if (end < 0 || begin < 0 || begin > end || dot > end) 
			return step; 
		
		//Set steps sub usecase's id to the one found within ( )
		step.setSubUseCaseId(line.substring(begin,end));		
		
		//Remove relation id from the step
		step.setStep(line.substring(0,begin-1));
		
		return step;
	}
	
	
	/**
	 * Tests if this adapter can read the file. For now, only test is that the file ends with ".txt"
	 * @param url url of the file to test
	 * @return true if this adapter can read the file
	 */
	public boolean canRead(URL url) {
		if (url.getFile().toLowerCase().endsWith(".txt")) 
			return true;
		return false;
	}
    
    /**
     * returns inputs name
     */
    public String toString() {
        return getName();
    }
	
	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#loadDefaultProperties()
	 */
	public Properties loadDefaultProperties() {
		Properties result = super.loadDefaultProperties();
		// TODO Set default properties here, for example:
		// result.setProperty("SETTING_1", "Woot");
		// result.setProperty("SETTING_2", "WTF?");
		return result;
	}

	/* (non-Javadoc)
	 * @see ucot.ModulePropertyInterface#applyProperties()
	 */
	public void applyProperties() throws BadPropertyValueException {
		// TODO Auto-generated method stub
		
	}
	
}
