package ucot.ui.gui.tree.usecasetree;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.filechooser.FileSystemView;

import ucot.input.UseCase;

/**
 * <pre>public class <b>FileTreeItem</b> <br />extends TreeItem</pre>
 * 
 * <p><code>TreeItem</code> to hold data of source containing usecases.</p>
 * 
 * @see #ucot.ui.gui.tree.usecasetree.TreeItem
 * 
 * @author ilanliuk
 * 
 */
public class FileTreeItem extends TreeItem {

    // Map for saving default filesystem icons for
    // file extensions, so we don't have to always
    // search them.
    private static Map<String, Icon> iconsForFileExtensions = new HashMap<String, Icon>();
    
    private URL url;
    
    /**
     * <p>Default constructor for <code>FileTreeItem</code>.</p>
     * 
     * @param treemodel <code>UseCaseTreeModel</code> wich uses this <code>FileTreeItem</code>.
     * @param parent <code>FilesTreeItem</code> wich is parent of this.
     * @param url <code>URL</code> to source this <code>FileTreeItem</code> contains.
     */
    public FileTreeItem(UseCaseTreeModel treemodel, TreeItem parent, URL url) {
        super(treemodel, parent);
        this.url = url;
    }

    @Override
    public String getName() {
        
        // return the filename from url (or the last path component)
        int i = url.getFile().lastIndexOf("/");
        return url.getFile().substring(i + 1);
    }
        
    @Override
    public List<TreeItem> getChildren() {
        List<TreeItem> list = new Vector<TreeItem>();
        List<UseCase> usecases = this.treemodel.getUseCaseCollection().getUseCasesFromURL(this.url);
        for (UseCase usecase : usecases) {
            list.add(new UseCaseTreeItem(this.treemodel, this, usecase));
        }
        
        return list;
    }

    @Override
    public Icon getIcon() {
        return getFilesystemIcon( this.getName() );
    }

    /**
     * <p>Returns file systems default icon for file described in string.</p>
     * 
     * <p>Ask file systems default <code>Icon</code> for given file
     * from <code>FileSystemView</code>.</p>
     * 
     * @param s name of the file we want to get <code>Icon</code> for.
     * @return <code>Icon</code> for given file.
     * 
     * @see FileSystemView
     */
    private static Icon getFilesystemIcon(String s) {
        
        // Strip extension from file
        String ext = s.substring(s.lastIndexOf("."));

        // See if have already added Icon for this
        // extension into map.
        Icon result = iconsForFileExtensions.get(ext);
        
        // if found icon, return it
        if (result != null) {
            return result;
        }
        
        // else try search for correct icon and
        // add it to the map
        try {
            
            // create tempfile with extension we just got
            File tempfile = File.createTempFile("UCOTtempfile", ext);
            
            // Get filesystemview and ask Icon for tempfile from it.
            FileSystemView filesystemview = FileSystemView.getFileSystemView();
            result = filesystemview.getSystemIcon(tempfile);
            
            // remove tempfile
            tempfile.delete();
            
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
        // if found icon for extension 
        // add it into map
        if (result != null) {
            iconsForFileExtensions.put(ext, result);
        }
        
        return result;
    }

    /**
     * <p>Returns <code>URL</code> of this <code>FileTreeItem</code>.</p>
     * 
     * @return <code>URL</code> of this <code>FileTreeItem</code>.
     */
    public URL getUrl() {
        return url;
    }
            
    
    
}
