package ucot.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Properties;

/**
 * Tools for properties management. Saving and loading from file and such.
 * 
 * @author UCOT
 */
public class PropertiesTools {
	
	private static final File propertiesFile = new File("ucot.xml");
	public static final URL propertiesURL = getPropertiesURL();
	
	/**
	 * Method for creating the URL from the properties file.
	 * @return URL to the properties file.
	 */
	private static URL getPropertiesURL() {
		try {
			return propertiesFile.toURL();
		} catch (MalformedURLException e) {
			return null;
		}
	}
	
	/**
	 * Method for merging two sets of properties in such a way where
	 * overriding properties' values will override any values in virtual
	 * properties set that have the same key as in overriding set.
	 * @param overriding Overriding properties set.
	 * @param virtual Virtual properties set.
	 * @return Merged properties set.
	 */
	public static Properties merge(Properties overriding, Properties virtual) {
		if ( overriding == null ) overriding = new Properties();
		if ( virtual == null ) virtual = new Properties();
		
		Properties result = new Properties();
		Enumeration keys = virtual.propertyNames();
		while ( keys.hasMoreElements() ) {
			String key = (String)keys.nextElement();
			result.setProperty(key, virtual.getProperty(key));
		}
		keys = overriding.propertyNames();
		while ( keys.hasMoreElements() ) {
			String key = (String)keys.nextElement();
			result.setProperty(key, overriding.getProperty(key));
		}
		return result;
	}
	
	/**
	 * Method for saving current settings to the properties XML file.
	 * Notice that only the values for keys given in properties will be changed
	 * and all other possible keys and values in propertiesFile will remain
	 * untouched. 
	 * @param properties Properties to be saved.
	 * @param propertiesURL Target file.
	 */
	public static void saveProperties(Properties properties, URL propertiesURL)
			throws IOException {
		try {
			FileOutputStream outputStream = null;
			
			// Load old values from file and merge to those.
			Properties oldProperties = null;
			try {
				oldProperties = PropertiesTools.loadProperties(null, propertiesURL);
			} catch (Exception e) {}
			properties = PropertiesTools.merge(properties, oldProperties);
			
			try {
				outputStream = new FileOutputStream(propertiesURL.getFile());
				properties.storeToXML(outputStream, "");
			} finally {
				if ( outputStream != null ) outputStream.close();
			}
		} catch (IOException e) {
			throw e;
		}
	}
	
	/**
	 * Method for (re)saving properties to the current properties XML file.
	 * @param properties Properties to be saved.
	 */
	public static void saveProperties(Properties properties) throws IOException {
		PropertiesTools.saveProperties(properties, PropertiesTools.propertiesURL);
	}
	
	/**
	 * Method for loading settings from the properties XML file.
	 * @param propertiesKeys Properties' keys that are supposed to be loaded.
	 *   If this is null, then all properties in the properties file are returned.
	 * @param propertiesURL Target file.
	 * @return Loaded properties.
	 */
	public static Properties loadProperties(Enumeration propertiesKeys, URL propertiesURL)
			throws IOException {
		FileInputStream inputStream = null;
		Properties result = new Properties();
		Properties loaded = new Properties();
		
		try {
			try {
				inputStream = new FileInputStream(propertiesURL.getFile());
				loaded.loadFromXML(inputStream);
			} finally {
				if ( inputStream != null ) inputStream.close();
			}
		} catch (IOException e) {
			throw e;
		}
		
		if ( propertiesKeys == null ) return loaded;
		
		while ( propertiesKeys.hasMoreElements() ) {
			String key = (String)propertiesKeys.nextElement();
			result.setProperty(key, (loaded.getProperty(key) != null ?
					loaded.getProperty(key) : ""));
		}
		
		return result;
	}
	
	/**
	 * Method for (re)loading settings from the current properties XML file.
	 * @param propertiesKeys Properties' keys that are supposed to be loaded.
	 * @return Loaded properties.
	 */
	public static Properties loadProperties(Enumeration propertiesKeys) throws IOException {
		return PropertiesTools.loadProperties(propertiesKeys, PropertiesTools.propertiesURL);
	}
}
